/**
 * Copyright (C) 1998-1999, Gopalan Suresh Raj.
 * All Rights Reserved.
 * 
 * This source code is part of a Java/JPE tutorial 
 * being developed by me.
 * 
 * See detailed instructions at my web site at
 * http://www.execpc.com/~gopalan
 * 
 */

package Bank;

import java.rmi.*;
import javax.ejb.*;
import java.util.*;
import java.sql.*;

public class CheckingBean implements SessionBean { 
 
 protected SessionContext _sessionContext;
 
 static final int ALL_FIELDS   = 0;
 static final int NAME_FIELD   = 1;
 static final int BALANCE_FIELD= 2;
 
 //////////////////////////////////////////////////////////////////// 
 //          Home Interface methods
 //////////////////////////////////////////////////////////////////// 
 //////////////////////////////////////////////////////////////////// 
 public void ejbCreate () {
  //////////////////////////////////////////////////////////////////// 
 }

 //////////////////////////////////////////////////////////////////// 
 //          Remote Interface methods
 //////////////////////////////////////////////////////////////////// 
 //////////////////////////////////////////////////////////////////// 
 public void createAccount (int key, String name, 
                            double startingBalance) {
  //////////////////////////////////////////////////////////////////// 
  if (startingBalance > 0) {
   truePut (CheckingBean.ALL_FIELDS, key, 
            startingBalance, name);
  }
 }

 //////////////////////////////////////////////////////////////////// 
 public void credit (double amount, int key) {
  //////////////////////////////////////////////////////////////////// 
  double balance = 0.0;
  
  try {
   balance = this.getBalance (key);
  }
  catch (Exception e) {
   e.printStackTrace ();
  }
  
  System.out.println("changing Checking::balance from " + balance);

  try {
   if( amount > 0 ) {
    balance += amount;
    truePut (CheckingBean.BALANCE_FIELD, key, 
             balance, "");
   }
  }
  catch (Exception e) {
   e.printStackTrace ();
  }
  System.out.println(" to " + balance);
 }
 
 //////////////////////////////////////////////////////////////////// 
 public void debit (double amount, int key) {
  //////////////////////////////////////////////////////////////////// 
  double balance = 0.0;
  
  try {
   balance = getBalance (key);
  }
  catch (Exception e) {
   e.printStackTrace ();
  }
  System.out.println("changing CheckingBean::balance from " + balance);
  
  try {
   if ( (amount > 0) && (balance >= amount) ) {
    balance -= amount;
    truePut (CheckingBean.BALANCE_FIELD, key, 
             balance, "");
   }
  }
  catch (Exception e) {
   e.printStackTrace ();
  }
  System.out.println(" to " + balance);
 }
 
 //////////////////////////////////////////////////////////////////// 
 public double getBalance (int key) {
  //////////////////////////////////////////////////////////////////// 
  double  balance = 0.0;

  try {
   
   String result = trueGet (CheckingBean.BALANCE_FIELD, key);
   balance = (new Double (result)).doubleValue();
  }
  catch (Exception e) {
   e.printStackTrace ();
  }
  
  System.out.println("CheckingBean::balance is " + balance);
  return balance;
 }
 
 //////////////////////////////////////////////////////////////////// 
 public String getCustomerName (int key) {
  //////////////////////////////////////////////////////////////////// 
  
  String  name = null;
  boolean bSuccess= false;
  
  try {
   name = trueGet (CheckingBean.NAME_FIELD, key);
  }
  catch (Exception e) {
   e.printStackTrace ();
  }
  
  System.out.println ("Checking::customer_name is " + name);
  return name;
 }

 //////////////////////////////////////////////////////////////////// 
 //          Other Internal methods
 //////////////////////////////////////////////////////////////////// 
 //////////////////////////////////////////////////////////////////// 
 public Connection getConnection () throws Exception {
 //////////////////////////////////////////////////////////////////// 
  return DriverManager.getConnection ("jdbc:odbc:Bank");
 }
 
 //////////////////////////////////////////////////////////////////// 
 String trueGet (int type, int key) {
  //////////////////////////////////////////////////////////////////// 
  Connection connection = null;
  PreparedStatement stmt= null;
  ResultSet  rset = null;
  String     result = null;
  String     query = null;
  String     fieldName = null;
  
  switch (type) {
   case CheckingBean.NAME_FIELD:
    query = "SELECT CUSTOMER_NAME FROM Checkings WHERE ACCOUNT_NUMBER = ?";
    fieldName = "CUSTOMER_NAME";
    break;

   case CheckingBean.BALANCE_FIELD:
    query = "SELECT BALANCE FROM Checkings WHERE ACCOUNT_NUMBER = ?";
    fieldName = "BALANCE";
    break;
  }
  
  try {
   //STEP1 : open the connection
   connection = this.getConnection ();
   stmt = connection.prepareStatement (query);
   stmt.setInt (1, key);
   
   //STEP 2: Obtain the desired resultset with a query
   rset = stmt.executeQuery ();
   
   //STEP 3:Get the appropriate fields
   if (!rset.wasNull ()) {
    rset.next ();
    result = rset.getString (fieldName);
   }
  }
  catch (Exception e) { 
   e.printStackTrace (); 
  }
  finally {
   // Cleanup that needs to occur whether we leave via a return or exception
   try {
    if (stmt != null) {
     stmt.close ();
     stmt = null;
    }
    if (connection != null) {
     connection.close ();
     connection = null;
    }
   }
   catch (Exception e) { 
    e.printStackTrace (); 
   }
  }
  return result; 
 }

 //////////////////////////////////////////////////////////////////// 
 String truePut (int type, int key, double balance, 
                 String name) {
  //////////////////////////////////////////////////////////////////// 
  Connection connection = null;
  PreparedStatement stmt= null;
  ResultSet  rset = null;
  String     result = null;
  String     query = null;
  
  switch (type) {
   case CheckingBean.BALANCE_FIELD:
    query = "UPDATE Checkings SET BALANCE = ? WHERE ACCOUNT_NUMBER = ?";
    break;

   case CheckingBean.NAME_FIELD:
   case CheckingBean.ALL_FIELDS:
   default:
    query = "INSERT INTO Checkings VALUES ( ?, ?, ? )";
    break;
  }
  
  try {
   //STEP1 : open the connection
   connection = this.getConnection ();
   stmt = connection.prepareStatement (query);
   if (type == CheckingBean.BALANCE_FIELD) {
    stmt.setDouble (1, balance);
    stmt.setInt (2, key);
   }
   else {
    stmt.setInt (1, key);
    stmt.setString (2, name);
    stmt.setDouble (3, balance);
   }
   
   //STEP 2: Obtain the desired resultset with a query
   rset = stmt.executeQuery ();
   rset.next ();
   
  }
  catch (Exception e) { 
   e.printStackTrace (); 
  }
  finally {
   // Cleanup that needs to occur whether we leave via a return or exception
   try {
    if (stmt != null) {
     stmt.close ();
     stmt = null;
    }
    if (connection != null) {
     connection.close ();
     connection = null;
    }
   }
   catch (Exception e) { 
    e.printStackTrace (); 
   }
  }
  return result; 
 }
 
 //////////////////////////////////////////////////////////////////// 
 public CheckingBean () {
  //////////////////////////////////////////////////////////////////// 
 }

 //////////////////////////////////////////////////////////////////// 
 //          Mandatory callback methods required by the EJB Spec
 //////////////////////////////////////////////////////////////////// 
 //////////////////////////////////////////////////////////////////// 
 public void ejbActivate () { 
  //////////////////////////////////////////////////////////////////// 
 }

 //////////////////////////////////////////////////////////////////// 
 public void ejbPassivate () { 
  //////////////////////////////////////////////////////////////////// 
 }

 //////////////////////////////////////////////////////////////////// 
 public void ejbRemove () { 
  //////////////////////////////////////////////////////////////////// 
 }

 //////////////////////////////////////////////////////////////////// 
 public void setSessionContext (SessionContext context) throws RemoteException { 
  //////////////////////////////////////////////////////////////////// 
  this._sessionContext = context;
 }

}

 